/*******************************************************************************
* FILE NAME: camera_turn.c
*
* DESCRIPTION:
*  Sample camera driven robot
*
*
*******************************************************************************/

#include <stdio.h>
#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "camera.h"
#include "tracking.h"

#include "auto_routines.h"
#include "camera_turn.h"


/*******************************************************************************
* FUNCTION NAME: Camera_Turn
* PURPOSE:       Turn the robot to face the target
*                Assume the servo powered gimbal is much faster than moving the whole robot
*******************************************************************************/
#define CENTER   127   // The servo setting when the robot is pointed at the target
#define KP     12/10   // Proportional gain
#define KI     5/100    // Integral gain (if a fraction then take care with MAXERROR)
#define DEADBAND 2
#define NEUTRAL  127
#define TARGET_CONFIDENCE_THRESHOLD 150
#define MAXERROR 300  // Maximum error we will allow to accumulate (MAXERROR < SPEED_LIMIT/KI)
#define SPEED_LIMIT 60

void Camera_Turn(int *done)
{
  static int cumerror=0;
  signed int pan_error;
  signed int p, i;
  signed int output;

  *done = FALSE;

  if (T_Packet_Data.confidence > TARGET_CONFIDENCE_THRESHOLD)  // First can we see the target?
  {
    pan_error = CENTER - (signed int)PAN_SERVO;  // How far off the target are we?
    if((pan_error <= DEADBAND) && (pan_error >= -DEADBAND) )
    { // We're there
	  pwm01 = NEUTRAL;
	  pwm02 = NEUTRAL;
      *done = TRUE;
      cumerror = 0;  // Can zero this to stop over-correcting
//      printf("We're done\r");
    }
    else    // Keep turning towards the light
    { 

      // Apply power Proportional to how far off we are
      p =  pan_error * KP;

      // Integral - Correct if we've been off by a little bit or if it's taking us too long
      i = cumerror * KI;
      cumerror += pan_error;
      if (cumerror > MAXERROR) cumerror = MAXERROR;
	  if (cumerror < -MAXERROR) cumerror = -MAXERROR;

      output = p + i;  // Calculate for motor forward
	  // Limit the motor output so it doesn't whip around too fast & isn't > char

      if (output > SPEED_LIMIT) output = SPEED_LIMIT;
      if (output < -SPEED_LIMIT) output = -SPEED_LIMIT;
      pwm01 = NEUTRAL - (unsigned char) output;
      pwm02 = NEUTRAL - (unsigned char) output; // The motors on one side are reversed
    }
//    printf("pan_error = %d, cum_pan_err, %d, PAN_SERVO = %d, pwm07=%d, pwm08=%d, output=%d\r",
//			(signed int)pan_error,cumerror,PAN_SERVO,pwm07,pwm08,output);
  }
  else  // Haven't found the target
  { // Wait for the camera to acquire the target (could spin the robot 360)
	pwm01 = NEUTRAL;
	pwm02 = NEUTRAL;	
//    printf("Haven't seen the target yet\r");
  }
}
