/*******************************************************************************
* FILE NAME: clock_routines.c
*
* DESCRIPTION:
*  This file contains the clock time keeping routines  
*  Uses system timer 4
*
* USAGE:
*  Does anybody really know what time it is?
*
*******************************************************************************/

#include "ifi_picdefs.h"
#include "timers.h"

#define VARIABLE_DECLARE  /* causes global variables to be declared only once */
  #include "clock_routines.h"
#undef VARIABLE_DECLARE


/*******************************************************************************
* FILE NAME: Initialize_Clock.c
*
* DESCRIPTION:
*  Initialize and start the internal timer for our clock  
*
* USAGE:
*  Called from User_Initialization()
*  8-bit timer
*  note: set timer to (255 - ticks)
*  time(ms) = ticks prescale postscale
*   0.1        250     4         1
*   0.2        125    16         1 
*   0.4        250    16         1
*   1          125    16         5
*   2          125    16        10
*   2          250    16         5
*   3          125    16        15
*   4          250    16        10
*   6          250    16        15
*
*******************************************************************************/

void Initialize_Clock(void)
{
/* Use Timer4 (8-bits) to control our clock */
/* timer set to 4 ms */

  OpenTimer4(TIMER_INT_ON &
             T4_PS_1_16 &
             T4_POST_1_10);

  WriteTimer4(6);  /* Preload timer to overflow after 4ms */

}
